import { Component, ElementRef, Input, Optional, Self, Inject, NgModule } from '@angular/core';
import { NgControl, NgForm, FormGroupDirective } from '@angular/forms';
import { ErrorStateMatcher } from '@angular/material/core';
import { MatFormFieldControl } from '@angular/material/form-field';
import { CodemirrorComponent, WA_CODEMIRROR_DEFAULTS, CodemirrorModule } from '@webacad/ng-codemirror';
import { Subject } from 'rxjs';
import 'codemirror';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
let nextUniqueId = 0;
class MatCodemirrorComponent extends CodemirrorComponent {
    /**
     * @param {?} el
     * @param {?} _defaultErrorStateMatcher
     * @param {?} _parentForm
     * @param {?} _parentFormGroup
     * @param {?} ngControl
     * @param {?} defaults
     */
    constructor(el, _defaultErrorStateMatcher, _parentForm, _parentFormGroup, ngControl, defaults) {
        super(el, defaults);
        this._defaultErrorStateMatcher = _defaultErrorStateMatcher;
        this._parentForm = _parentForm;
        this._parentFormGroup = _parentFormGroup;
        this.ngControl = ngControl;
        this.required = false;
        this.errorState = false;
        this.stateChanges = new Subject();
        this.shouldLabelFloat = true;
        this._disabled = false;
        this._uid = `wa-codemirror-${nextUniqueId++}`;
        if (this.ngControl !== null) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    get value() {
        return this.codemirror ? this.codemirror.getValue() : '';
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set value(value) {
        if (this.codemirror) {
            if (value !== this.value) {
                this.codemirror.setValue(value);
                this.stateChanges.next();
            }
        }
    }
    /**
     * @return {?}
     */
    get disabled() {
        if (this.ngControl && this.ngControl.disabled !== null) {
            return this.ngControl.disabled;
        }
        return this._disabled;
    }
    /**
     * @param {?} disabled
     * @return {?}
     */
    set disabled(disabled) {
        this._disabled = disabled;
    }
    /**
     * @return {?}
     */
    get id() {
        return this._id;
    }
    /**
     * @param {?} id
     * @return {?}
     */
    set id(id) {
        this._id = id || this._uid;
    }
    /**
     * @return {?}
     */
    get empty() {
        return this.value !== '';
    }
    /**
     * @return {?}
     */
    get focused() {
        if (this.codemirror) {
            return this.codemirror.hasFocus();
        }
        return false;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        super.ngOnChanges(changes);
        if (typeof changes['value'] !== 'undefined' || typeof changes['required'] !== 'undefined') {
            this.stateChanges.next();
        }
    }
    /**
     * @return {?}
     */
    ngDoCheck() {
        if (this.ngControl) {
            this.updateErrorState();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.stateChanges.complete();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onContainerClick(event) {
        this.focus();
    }
    /**
     * @return {?}
     */
    focus() {
        if (this.codemirror) {
            this.codemirror.focus();
            this.stateChanges.next();
        }
    }
    /**
     * @return {?}
     */
    updateErrorState() {
        const /** @type {?} */ oldState = this.errorState;
        const /** @type {?} */ parent = this._parentFormGroup || this._parentForm;
        const /** @type {?} */ matcher = this.errorStateMatcher || this._defaultErrorStateMatcher;
        const /** @type {?} */ control = this.ngControl ? /** @type {?} */ (this.ngControl.control) : null;
        const /** @type {?} */ newState = matcher.isErrorState(control, parent);
        if (newState !== oldState) {
            this.errorState = newState;
            this.stateChanges.next();
        }
    }
    /**
     * @param {?} ids
     * @return {?}
     */
    setDescribedByIds(ids) {
    }
}
MatCodemirrorComponent.decorators = [
    { type: Component, args: [{
                selector: 'wa-mat-codemirror',
                template: '',
                providers: [
                    {
                        provide: MatFormFieldControl,
                        useExisting: MatCodemirrorComponent,
                    },
                ],
            },] },
];
/** @nocollapse */
MatCodemirrorComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: ErrorStateMatcher, },
    { type: NgForm, decorators: [{ type: Optional },] },
    { type: FormGroupDirective, decorators: [{ type: Optional },] },
    { type: NgControl, decorators: [{ type: Self }, { type: Optional },] },
    { type: undefined, decorators: [{ type: Inject, args: [WA_CODEMIRROR_DEFAULTS,] }, { type: Optional },] },
];
MatCodemirrorComponent.propDecorators = {
    "placeholder": [{ type: Input },],
    "required": [{ type: Input },],
    "errorStateMatcher": [{ type: Input },],
    "value": [{ type: Input },],
    "disabled": [{ type: Input },],
    "id": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MatCodemirrorModule {
}
MatCodemirrorModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CodemirrorModule,
                ],
                declarations: [
                    MatCodemirrorComponent,
                ],
                exports: [
                    CodemirrorModule,
                    MatCodemirrorComponent,
                ],
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { MatCodemirrorComponent, MatCodemirrorModule };
//# sourceMappingURL=webacad-ng-mat-codemirror.js.map
